var _a;
import { RawAccount, cojsonInternals, } from "cojson";
import { inspect, } from "../../coValueInterfaces.js";
import { SharedCoValueConstructor } from "../construction.js";
import { AST, Schema } from "@effect/schema";
import { constructorOfSchemaSym, propertyIsCoValueSchema, } from "../resolution.js";
import { pipeArguments } from "effect/Pipeable";
import { ValueRef } from "../../refs.js";
import { Account, controlledAccountFromNode } from "../account/accountOf.js";
import { Group } from "../group/groupOf.js";
export function CoStreamOf(itemSchema) {
    const decodeItem = Schema.decodeSync(itemSchema);
    const encodeItem = Schema.encodeSync(itemSchema);
    const itemIsCoValue = propertyIsCoValueSchema(itemSchema);
    class CoStreamOfItem extends SharedCoValueConstructor {
        static get ast() {
            return AST.setAnnotation(Schema.instanceOf(this).ast, constructorOfSchemaSym, this);
        }
        static pipe() {
            // eslint-disable-next-line prefer-rest-params
            return pipeArguments(this, arguments);
        }
        constructor(init, options) {
            super();
            let raw;
            if ("fromRaw" in options) {
                raw = options.fromRaw;
            }
            else {
                const rawOwner = options.owner._raw;
                raw = rawOwner.createStream();
            }
            this.by = {};
            this.in = {};
            Object.defineProperties(this, {
                id: { value: raw.id, enumerable: false },
                _type: { value: "CoStream", enumerable: false },
                _owner: {
                    get: () => raw.group instanceof RawAccount
                        ? Account.fromRaw(raw.group)
                        : Group.fromRaw(raw.group),
                    enumerable: false,
                },
                _raw: { value: raw, enumerable: false },
                _loadedAs: {
                    get: () => controlledAccountFromNode(raw.core.node),
                    enumerable: false,
                },
                _schema: { value: this.constructor, enumerable: false },
            });
            if (init !== undefined) {
                for (const item of init) {
                    this.pushItem(item);
                }
            }
            this.updateEntries();
        }
        updateEntries() {
            // eslint-disable-next-line @typescript-eslint/no-this-alias
            const self = this;
            const raw = this._raw;
            const loadedAs = this._loadedAs;
            for (const accountID of this._raw.accounts()) {
                Object.defineProperty(this.by, accountID, {
                    get() {
                        const rawEntry = raw.lastItemBy(accountID);
                        if (!rawEntry)
                            return;
                        return entryFromRawEntry(self, rawEntry, loadedAs, accountID);
                    },
                    configurable: true,
                });
            }
            Object.defineProperty(this.by, "me", {
                get() {
                    return self.by[loadedAs.id];
                },
                enumerable: false,
                configurable: true,
            });
            for (const sessionID of raw.sessions()) {
                Object.defineProperty(this.in, sessionID, {
                    get() {
                        const rawEntry = raw.lastItemIn(sessionID);
                        if (!rawEntry)
                            return;
                        const by = cojsonInternals.accountOrAgentIDfromSessionID(sessionID);
                        return entryFromRawEntry(self, rawEntry, loadedAs, cojsonInternals.isAccountID(by)
                            ? by
                            : undefined);
                    },
                    configurable: true,
                });
            }
            Object.defineProperty(this.in, "currentSession", {
                get() {
                    return self.in[loadedAs.sessionID];
                },
                enumerable: false,
                configurable: true,
            });
        }
        static fromRaw(raw) {
            return new CoStreamOfItem(undefined, { fromRaw: raw });
        }
        push(...items) {
            for (const item of items) {
                this.pushItem(item);
            }
            this.updateEntries();
        }
        pushItem(item) {
            if (itemIsCoValue) {
                this._raw.push(item.id);
            }
            else {
                this._raw.push(encodeItem(item));
            }
        }
        toJSON() {
            return {
                by: Object.fromEntries(Object.entries(this.by).map(([key, value]) => [
                    key,
                    value &&
                        typeof value === "object" &&
                        "toJSON" in value &&
                        typeof value.toJSON === "function"
                        ? value.toJSON()
                        : value,
                ])),
                in: Object.fromEntries(Object.entries(this.in).map(([key, value]) => [
                    key,
                    value &&
                        typeof value === "object" &&
                        "toJSON" in value &&
                        typeof value.toJSON === "function"
                        ? value.toJSON()
                        : value,
                ])),
                co: {
                    id: this.id,
                    type: "CoStream",
                },
            };
        }
        [(Schema.TypeId, inspect)]() {
            return this.toJSON();
        }
        static as() {
            return CoStreamOfItem;
        }
    }
    CoStreamOfItem.type = "CoStream";
    return CoStreamOfItem;
    function entryFromRawEntry(accessFrom, rawEntry, loadedAs, accountID) {
        return {
            get value() {
                if (itemIsCoValue) {
                    return this.ref?.accessFrom(accessFrom);
                }
                else {
                    return decodeItem(rawEntry.value);
                }
            },
            get ref() {
                if (itemIsCoValue) {
                    const rawId = rawEntry.value;
                    return new ValueRef(rawId, loadedAs, itemSchema);
                }
            },
            get by() {
                return (accountID &&
                    new ValueRef(accountID, loadedAs, Account)?.accessFrom(accessFrom));
            },
            madeAt: rawEntry.at,
            tx: rawEntry.tx,
        };
    }
}
class BinaryCoStreamImplClass extends SharedCoValueConstructor {
    static pipe() {
        // eslint-disable-next-line prefer-rest-params
        return pipeArguments(this, arguments);
    }
    constructor(init, options) {
        super();
        let raw;
        if ("fromRaw" in options) {
            raw = options.fromRaw;
        }
        else {
            const rawOwner = options.owner._raw;
            raw = rawOwner.createBinaryStream();
        }
        Object.defineProperties(this, {
            id: { value: raw.id, enumerable: false },
            _type: { value: "BinaryCoStream", enumerable: false },
            _owner: {
                get: () => raw.group instanceof RawAccount
                    ? Account.fromRaw(raw.group)
                    : Group.fromRaw(raw.group),
                enumerable: false,
            },
            _raw: { value: raw, enumerable: false },
            _loadedAs: {
                get: () => controlledAccountFromNode(raw.core.node),
                enumerable: false,
            },
            _schema: { value: this.constructor, enumerable: false },
        });
    }
    static fromRaw(raw) {
        return new _a(undefined, { fromRaw: raw });
    }
    getChunks(options) {
        return this._raw.getBinaryChunks(options?.allowUnfinished);
    }
    start(options) {
        this._raw.startBinaryStream(options);
    }
    push(data) {
        this._raw.pushBinaryStreamChunk(data);
    }
    end() {
        this._raw.endBinaryStream();
    }
    toJSON() {
        return {
            ...this.getChunks(),
            co: {
                id: this.id,
                type: "BinaryCoStream",
            },
        };
    }
    [(Schema.TypeId, inspect)]() {
        return this.toJSON();
    }
}
_a = BinaryCoStreamImplClass;
BinaryCoStreamImplClass.ast = AST.setAnnotation(Schema.instanceOf(_a).ast, constructorOfSchemaSym, _a);
BinaryCoStreamImplClass.type = "BinaryCoStream";
export const BinaryCoStreamImpl = BinaryCoStreamImplClass;
//# sourceMappingURL=coStreamOf.js.map