import { Effect, Sink, Stream } from "effect";
import { ControlledAccountCtx } from "./account/account.js";
import { ValueRef } from "../refs.js";
import { SubscriptionScope } from "../subscriptionScope.js";
export class SharedCoValueConstructor {
    static loadEf(id) {
        return Effect.gen(this, function* (_) {
            const controlledAccount = yield* _(ControlledAccountCtx);
            return yield* _(new ValueRef(id, controlledAccount, this).loadEf());
        });
    }
    static load(id, options) {
        return new ValueRef(id, options.as, this).load(options?.onProgress && { onProgress: options.onProgress });
    }
    static subscribe(id, options, onUpdate) {
        void Effect.runPromise(Effect.provideService(this.subscribeEf(id).pipe(Stream.run(Sink.forEach((update) => Effect.sync(() => onUpdate(update))))), ControlledAccountCtx, options.as));
        return function unsubscribe() { };
    }
    static subscribeEf(id) {
        return Stream.fromEffect(this.loadEf(id)).pipe(Stream.flatMap((value) => Stream.asyncScoped((emit) => Effect.gen(this, function* (_) {
            const subscription = new SubscriptionScope(value, this, (update) => {
                void emit.single(update);
            });
            yield* _(Effect.addFinalizer(() => Effect.sync(() => subscription.unsubscribeAll())));
        }))));
    }
    subscribe(listener) {
        return this._schema.subscribe(this.id, { as: this._loadedAs }, listener);
    }
    subscribeEf() {
        return Stream.provideService(this._schema.subscribeEf(this.id), ControlledAccountCtx, this._loadedAs);
    }
}
//# sourceMappingURL=construction.js.map