import { inspect, isCoValueSchema, } from "../../coValueInterfaces.js";
import * as S from "@effect/schema/Schema";
import { isControlledAccount, } from "../account/account.js";
import { AST, Schema } from "@effect/schema";
import { constructorOfSchemaSym } from "../resolution.js";
import { pipeArguments } from "effect/Pipeable";
import { ValueRef } from "../../refs.js";
import { controlledAccountFromNode } from "../account/accountOf.js";
import { SharedCoValueConstructor } from "../construction.js";
export function GroupOf(fields) {
    class GroupOfProfileAndRoot extends SharedCoValueConstructor {
        static get ast() {
            return AST.setAnnotation(Schema.instanceOf(this).ast, constructorOfSchemaSym, this);
        }
        static pipe() {
            // eslint-disable-next-line prefer-rest-params
            return pipeArguments(this, arguments);
        }
        constructor(init, options) {
            super();
            let raw;
            if (options && "fromRaw" in options) {
                raw = options.fromRaw;
            }
            else {
                const initOwner = options?.owner || init?.owner;
                if (!initOwner)
                    throw new Error("No owner provided");
                if (isControlledAccount(initOwner)) {
                    const rawOwner = initOwner._raw;
                    raw = rawOwner.createGroup();
                }
                else {
                    throw new Error("Can only construct group as a controlled account");
                }
            }
            const refs = {
                get profile() {
                    if (isCoValueSchema(fields.profile)) {
                        const profileID = raw.get("profile");
                        return (profileID &&
                            new ValueRef(profileID, controlledAccountFromNode(raw.core.node), fields.profile));
                    }
                },
                get root() {
                    if (isCoValueSchema(fields.root)) {
                        const rootID = raw.get("root");
                        return (rootID &&
                            new ValueRef(rootID, controlledAccountFromNode(raw.core.node), fields.root));
                    }
                },
            };
            Object.defineProperties(this, {
                id: { value: raw.id, enumerable: false },
                _type: { value: "Group", enumerable: false },
                _owner: { value: this, enumerable: false },
                _refs: { value: refs, enumerable: false },
                _raw: { value: raw, enumerable: false },
                _loadedAs: {
                    get: () => controlledAccountFromNode(raw.core.node),
                    enumerable: false,
                },
                _schema: { value: GroupOfProfileAndRoot, enumerable: false },
            });
        }
        static fromRaw(raw) {
            return new GroupOfProfileAndRoot(undefined, {
                fromRaw: raw,
            });
        }
        get profile() {
            return this._refs.profile.accessFrom(this);
        }
        get root() {
            return this._refs.root.accessFrom(this);
        }
        addMember(member, role) {
            this._raw.addMember(typeof member === "string" ? member : member._raw, role);
            return this;
        }
        myRole() {
            return this._raw.myRole();
        }
        toJSON() {
            return {
                co: {
                    id: this.id,
                    type: this._type,
                },
                profile: this.profile?.toJSON(),
                root: this.root?.toJSON(),
            };
        }
        [(Schema.TypeId, inspect)]() {
            return this.toJSON();
        }
        static as() {
            return this;
        }
    }
    GroupOfProfileAndRoot.type = "Group";
    return GroupOfProfileAndRoot;
}
export class Group extends GroupOf({
    profile: S.null,
    root: S.null,
}).as() {
}
//# sourceMappingURL=groupOf.js.map