import { Effect } from "effect";
import { getCoValueConstructorInProperty } from "./coValues/resolution.js";
import { subscriptionsScopes } from "./subscriptionScope.js";
export class ValueRef {
    constructor(id, controlledAccount, propDef) {
        this.id = id;
        this.controlledAccount = controlledAccount;
        this.propDef = propDef;
    }
    get value() {
        if (this.cachedValue)
            return this.cachedValue;
        // TODO: cache it for object identity!!!
        const raw = this.controlledAccount._raw.core.node.getLoaded(this.id);
        if (raw) {
            const Constructor = getCoValueConstructorInProperty(this.propDef, raw);
            if (!Constructor) {
                throw new Error("Couldn't extract CoValue constructor from property definition");
            }
            const value = new Constructor(undefined, { fromRaw: raw });
            this.cachedValue = value;
            return value;
        }
    }
    loadEf() {
        return Effect.async((fulfill) => {
            this.loadHelper()
                .then((value) => {
                if (value === "unavailable") {
                    fulfill(Effect.fail("unavailable"));
                }
                else {
                    fulfill(Effect.succeed(value));
                }
            })
                .catch((e) => {
                fulfill(Effect.die(e));
            });
        });
    }
    async loadHelper(options) {
        const raw = await this.controlledAccount._raw.core.node.load(this.id, options?.onProgress);
        if (raw === "unavailable") {
            return "unavailable";
        }
        else {
            return new ValueRef(this.id, this.controlledAccount, this.propDef)
                .value;
        }
    }
    async load(options) {
        const result = await this.loadHelper(options);
        if (result === "unavailable") {
            return undefined;
        }
        else {
            return result;
        }
    }
    accessFrom(fromScopeValue) {
        const subScope = subscriptionsScopes.get(fromScopeValue);
        subScope?.onRefAccessedOrSet(this.id);
        if (this.value && subScope) {
            subscriptionsScopes.set(this.value, subScope);
        }
        return this.value;
    }
}
export function makeRefs(getIdForKey, getKeysWithIds, controlledAccount, propDefForKey) {
    const refs = {};
    return new Proxy(refs, {
        get(target, key) {
            if (key === Symbol.iterator) {
                return function* () {
                    for (const key of getKeysWithIds()) {
                        yield new ValueRef(getIdForKey(key), controlledAccount, propDefForKey(key));
                    }
                };
            }
            if (typeof key === "symbol")
                return undefined;
            if (key === "length") {
                return getKeysWithIds().length;
            }
            const id = getIdForKey(key);
            if (!id)
                return undefined;
            return new ValueRef(id, controlledAccount, propDefForKey(key));
        },
        ownKeys() {
            return getKeysWithIds().map((key) => key.toString());
        },
    });
}
//# sourceMappingURL=refs.js.map